import tkinter as tk
from tkinter import scrolledtext, messagebox, filedialog
import os
import re
from gtts import gTTS
import threading
from googletrans import Translator, LANGUAGES
import io
from pydub import AudioSegment
import winreg

class TextToSpeechApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Text zu Sprache Konverter")
        self.root.geometry("600x620")
        
        self.translator = Translator()
        
        self.bg_color = "#e0ebe0"
        self.button_color = "#5d937a"
        self.text_bg = "#f0f5f0"
        self.text_fg = "#2c503e"
        self.accent_color = "#4a7a5e"
        
        self.root.configure(bg=self.bg_color)
        
        self.languages = {
            "de": "Deutsch", "en-us": "US-Englisch", "en-gb": "Britisches Englisch", 
            "sv": "Schwedisch", "fr": "Französisch", "es": "Spanisch"
        }
        
        self.tts_to_translate_map = {
            "de": {"lang": "de", "tld": "de"}, "en-us": {"lang": "en", "tld": "com"},
            "en-gb": {"lang": "en", "tld": "co.uk"}, "sv": {"lang": "sv", "tld": "se"},
            "fr": {"lang": "fr", "tld": "fr"}, "es": {"lang": "es", "tld": "es"}
        }
        
        self.selected_language = tk.StringVar(value="de")
        self.slow_speech = tk.BooleanVar(value=False)
        self.auto_translate = tk.BooleanVar(value=False)
        self.original_text = ""
        self.output_dir = self.get_desktop_path()
        
        self.create_widgets()
    
    def get_desktop_path(self):
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
            return winreg.QueryValueEx(key, "Desktop")[0]
        except Exception:
            return os.path.join(os.path.expanduser('~'), 'Desktop')
    
    def get_downloads_path(self):
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
            return winreg.QueryValueEx(key, "{374DE290-123F-4565-9164-39C4925E467B}")[0]
        except Exception:
            return os.path.join(os.path.expanduser('~'), 'Downloads')
    
    def create_widgets(self):
        filename_frame = tk.Frame(self.root, bg=self.bg_color)
        filename_frame.pack(pady=10, fill=tk.X, padx=20)
        
        tk.Label(filename_frame, text="Dateiname:", bg=self.bg_color, fg=self.text_fg, font=("Arial", 10, "bold")).pack(side=tk.LEFT, padx=(0, 10))
        self.filename_entry = tk.Entry(filename_frame, bg=self.text_bg, fg=self.text_fg, width=30)
        self.filename_entry.pack(side=tk.LEFT, fill=tk.X, expand=True)
        self.filename_entry.insert(0, "output")
        self.output_dir_button = tk.Button(filename_frame, text="Ordner", command=self.select_output_dir, bg=self.button_color, fg="white", font=("Arial", 9))
        self.output_dir_button.pack(side=tk.LEFT, padx=5)
        
        location_frame = tk.Frame(self.root, bg=self.bg_color)
        location_frame.pack(pady=0, fill=tk.X, padx=20)
        tk.Label(location_frame, text="Speicherort:", bg=self.bg_color, fg=self.text_fg, font=("Arial", 9)).pack(side=tk.LEFT, padx=(0, 5))
        self.location_label = tk.Label(location_frame, text=self.output_dir, bg=self.bg_color, fg=self.accent_color, font=("Arial", 9))
        self.location_label.pack(side=tk.LEFT, fill=tk.X)
        
        location_options_frame = tk.Frame(self.root, bg=self.bg_color)
        location_options_frame.pack(pady=2, fill=tk.X, padx=20)
        self.desktop_button = tk.Button(location_options_frame, text="Desktop", command=lambda: self.set_standard_dir("desktop"), bg=self.button_color, fg="white", font=("Arial", 9))
        self.desktop_button.pack(side=tk.LEFT, padx=(0, 5))
        self.downloads_button = tk.Button(location_options_frame, text="Downloads", command=lambda: self.set_standard_dir("downloads"), bg=self.button_color, fg="white", font=("Arial", 9))
        self.downloads_button.pack(side=tk.LEFT)
        
        language_frame = tk.Frame(self.root, bg=self.bg_color)
        language_frame.pack(pady=5, fill=tk.X, padx=20)
        tk.Label(language_frame, text="Sprache:", bg=self.bg_color, fg=self.text_fg, font=("Arial", 10, "bold")).pack(side=tk.LEFT, padx=(0, 10))
        
        language_buttons_frame = tk.Frame(self.root, bg=self.bg_color)
        language_buttons_frame.pack(pady=0, fill=tk.X, padx=20)
        col = 0
        row = 0
        for lang_code, lang_name in self.languages.items():
            rb = tk.Radiobutton(language_buttons_frame, text=lang_name, value=lang_code, variable=self.selected_language, bg=self.bg_color, fg=self.text_fg, command=self.on_language_change)
            rb.grid(row=row, column=col, sticky=tk.W, padx=5, pady=2)
            col += 1
            if col > 2:
                col = 0
                row += 1
        
        option_frame = tk.Frame(language_buttons_frame, bg=self.bg_color)
        option_frame.grid(row=row+1, column=0, columnspan=3, sticky=tk.W, padx=5, pady=5)
        slow_check = tk.Checkbutton(option_frame, text="Langsames Sprechen", variable=self.slow_speech, bg=self.bg_color, fg=self.text_fg)
        slow_check.pack(side=tk.LEFT, padx=(0, 15))
        translate_check = tk.Checkbutton(option_frame, text="Automatisch übersetzen", variable=self.auto_translate, bg=self.bg_color, fg=self.text_fg)
        translate_check.pack(side=tk.LEFT)
        
        tk.Label(self.root, text="Text eingeben:", bg=self.bg_color, fg=self.text_fg, font=("Arial", 10, "bold")).pack(anchor="w", padx=20)
        self.text_area = scrolledtext.ScrolledText(self.root, wrap=tk.WORD, width=60, height=12, bg=self.text_bg, fg=self.text_fg)
        self.text_area.pack(padx=20, pady=10, fill=tk.BOTH, expand=True)
        
        tk.Label(self.root, text="Übersetzter Text:", bg=self.bg_color, fg=self.text_fg, font=("Arial", 10, "bold")).pack(anchor="w", padx=20)
        self.translated_text_area = scrolledtext.ScrolledText(self.root, wrap=tk.WORD, width=60, height=5, bg=self.text_bg, fg=self.accent_color)
        self.translated_text_area.pack(padx=20, pady=10, fill=tk.X)
        
        button_frame = tk.Frame(self.root, bg=self.bg_color)
        button_frame.pack(pady=15, padx=20)
        self.translate_button = tk.Button(button_frame, text="Text übersetzen", command=self.translate_text, bg=self.button_color, fg="white", font=("Arial", 10, "bold"), activebackground="#49617a", width=15, height=2)
        self.translate_button.pack(side=tk.LEFT, padx=(0, 10))
        self.convert_button = tk.Button(button_frame, text="WAV erzeugen", command=self.start_conversion, bg=self.button_color, fg="white", font=("Arial", 10, "bold"), activebackground="#49617a", width=15, height=2)
        self.convert_button.pack(side=tk.LEFT, padx=(0, 10))
        self.clear_button = tk.Button(button_frame, text="Text löschen", command=self.clear_text, bg=self.button_color, fg="white", font=("Arial", 10, "bold"), activebackground="#49617a", width=15, height=2)
        self.clear_button.pack(side=tk.LEFT)
        
        self.status_label = tk.Label(self.root, text="", bg=self.bg_color, fg=self.text_fg)
        self.status_label.pack(pady=5)
    
    def select_output_dir(self):
        dir_path = filedialog.askdirectory(initialdir=self.output_dir)
        if dir_path:
            self.output_dir = dir_path
            self.location_label.config(text=self.output_dir)
    
    def set_standard_dir(self, dir_type):
        self.output_dir = self.get_desktop_path() if dir_type == "desktop" else self.get_downloads_path()
        self.location_label.config(text=self.output_dir)

    def clean_text(self, text):
        invalid_chars = r"[\"'""''`´…\-–—:;(){}\[\]<>|@#$%^&*_+=/\\😊😂👍]"
        return re.sub(invalid_chars, ' ', text)
    
    def on_language_change(self):
        if self.auto_translate.get() and self.original_text:
            self.translate_text()
    
    def translate_text(self):
        text = self.text_area.get("1.0", tk.END).strip()
        if not text:
            self.status_label.config(text="Fehler: Kein Text eingegeben!")
            return
        
        self.original_text = text
        selected_lang = self.selected_language.get()
        translate_lang = self.tts_to_translate_map[selected_lang]["lang"]
        
        self.translate_button.config(state=tk.DISABLED)
        self.status_label.config(text="Übersetze...")
        threading.Thread(target=self.perform_translation, args=(text, translate_lang), daemon=True).start()
    
    def perform_translation(self, text, dest_lang):
        try:
            translation = self.translator.translate(text, dest=dest_lang)
            self.translated_text_area.delete("1.0", tk.END)
            self.translated_text_area.insert("1.0", translation.text)
            self.status_label.config(text=f"Übersetzung von {LANGUAGES.get(translation.src, 'Unbekannt')} nach {LANGUAGES.get(translation.dest, 'Unbekannt')} abgeschlossen")
        except Exception as e:
            self.status_label.config(text=f"Fehler bei der Übersetzung: {str(e)}")
            self.translated_text_area.delete("1.0", tk.END)
            self.translated_text_area.insert("1.0", "Übersetzungsfehler. Bitte erneut versuchen.")
        finally:
            self.translate_button.config(state=tk.NORMAL)
    
    def convert_to_speech(self):
        text = self.translated_text_area.get("1.0", tk.END).strip() if self.auto_translate.get() else self.text_area.get("1.0", tk.END).strip()
        if not text and self.auto_translate.get():
            self.translate_text()
            text = self.translated_text_area.get("1.0", tk.END).strip()
        if not text:
            self.status_label.config(text="Fehler: Kein Text eingegeben!")
            return
        
        filename = self.filename_entry.get().strip() or "output"
        output_wav = os.path.join(self.output_dir, f"{filename}.wav")
        
        try:
            cleaned_text = self.clean_text(text)
            if not cleaned_text:
                self.status_label.config(text="Fehler: Der Text ist nach der Bereinigung leer!")
                return
            
            selected_lang = self.selected_language.get()
            lang_settings = self.tts_to_translate_map[selected_lang]
            is_slow = self.slow_speech.get()
            
            debug_msg = f"Erzeuge TTS mit lang={lang_settings['lang']}, tld={lang_settings['tld']}, slow={is_slow}"
            print(debug_msg)
            self.status_label.config(text=debug_msg)
            
            tts = gTTS(text=cleaned_text, lang=lang_settings["lang"], tld=lang_settings["tld"], slow=is_slow)
            mp3_fp = io.BytesIO()
            tts.write_to_fp(mp3_fp)
            mp3_fp.seek(0)
            
            sound = AudioSegment.from_mp3(mp3_fp)
            sound = sound.set_frame_rate(16000)  # Frühzeitige Reduktion der Abtastrate
            sound = sound.low_pass_filter(3500)  # Tiefpassfilter bei 3500 Hz
            sound = sound.set_frame_rate(8000).set_channels(1)  # Für Asterisk
            sound.export(output_wav, format="wav")
            
            lang_name = self.languages[selected_lang]
            speed_text = "langsam" if is_slow else "normal"
            translate_text = "übersetzt" if self.auto_translate.get() else "original"
            self.status_label.config(text=f"Erfolgreich erstellt: {output_wav} ({lang_name}, {speed_text}, {translate_text}, Tiefpass 3500 Hz)")
            os.startfile(self.output_dir)
        except Exception as e:
            self.status_label.config(text=f"Fehler: {str(e)}")
        finally:
            self.convert_button.config(state=tk.NORMAL)
    
    def start_conversion(self):
        self.convert_button.config(state=tk.DISABLED)
        self.status_label.config(text="Konvertiere...")
        if self.auto_translate.get() and not self.translated_text_area.get("1.0", tk.END).strip():
            self.translate_text()
        threading.Thread(target=self.convert_to_speech, daemon=True).start()
    
    def clear_text(self):
        self.text_area.delete("1.0", tk.END)
        self.translated_text_area.delete("1.0", tk.END)
        self.original_text = ""
        self.status_label.config(text="Text gelöscht")

def main():
    root = tk.Tk()
    app = TextToSpeechApp(root)
    root.mainloop()

if __name__ == "__main__":
    main()
